/*
 * Copyright (C) BABEC. All rights reserved.
 * Copyright (C) THL A29 Limited, a Tencent company. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

package testcontract

import (
	"chainmaker.org/chainmaker/vm-native/v2/common"

	"chainmaker.org/chainmaker/pb-go/v2/syscontract"
	"chainmaker.org/chainmaker/protocol/v2"
)

var (
	ContractName = syscontract.SystemContract_T.String()
)

type Manager struct {
	methods map[string]common.ContractFunc
	log     protocol.Logger
}

func NewManager(log protocol.Logger) *Manager {
	return &Manager{
		log:     log,
		methods: registerTestContractMethods(log),
	}
}

func (c *Manager) GetMethod(methodName string) common.ContractFunc {
	return c.methods[methodName]
}

func registerTestContractMethods(log protocol.Logger) map[string]common.ContractFunc {
	methodMap := make(map[string]common.ContractFunc, 64)
	runtime := &ManagerRuntime{log: log}
	methodMap[syscontract.TestContractFunction_P.String()] = common.WrapResultFunc(runtime.put)
	methodMap[syscontract.TestContractFunction_G.String()] = common.WrapResultFunc(runtime.get)
	methodMap[syscontract.TestContractFunction_D.String()] = common.WrapResultFunc(runtime.del)
	methodMap[syscontract.TestContractFunction_N.String()] = common.WrapResultFunc(runtime.nothing)
	return methodMap
}

type ManagerRuntime struct {
	log protocol.Logger
}

func (r *ManagerRuntime) get(context protocol.TxSimContext, parameters map[string][]byte) ([]byte, error) {
	return context.Get(ContractName, parameters["k"])
}
func (r *ManagerRuntime) put(context protocol.TxSimContext, parameters map[string][]byte) ([]byte, error) {
	k := parameters["k"]
	v := parameters["v"]
	return nil, context.Put(ContractName, k, v)
}

func (r *ManagerRuntime) del(context protocol.TxSimContext, parameters map[string][]byte) ([]byte, error) {
	return nil, context.Del(ContractName, parameters["k"])
}
func (r *ManagerRuntime) nothing(txSimContext protocol.TxSimContext, parameters map[string][]byte) (
	[]byte, error) {
	return nil, nil
}
